use std::sync::LazyLock;

use types::{
    ChatCompletionChoice, ChatCompletionMessage, ChatCompletionRequest, ChatCompletionResponse,
    ChatCompletionUsage, ListModelsResponse,
};

use crate::config::CONFIG;

pub mod types;

pub struct Groq {
    api_key: String,
    url: String,
    http_client: reqwest::Client,
}

impl Groq {
    pub fn new(api_key: String) -> Self {
        Self {
            api_key,
            url: "https://api.groq.com/openai/v1".to_string(),
            http_client: reqwest::Client::new(),
        }
    }

    pub async fn list_models(&self) -> Vec<String> {
        let response = self
            .http_client
            .get(format!("{}/models", self.url))
            .header("Authorization", format!("Bearer {}", self.api_key))
            .send()
            .await;

        if response.is_err() {
            print!("Request Error");
            return vec![];
        }

        let response_models = response.unwrap().json::<ListModelsResponse>().await;

        if response_models.is_err() {
            print!("Response Parsing Error");
            return vec![];
        }

        let model_names = response_models
            .unwrap()
            .data
            .iter()
            .map(|model| model.id.clone())
            .collect();

        model_names
    }

    pub async fn chat_completion(&self, model: String, message: String) -> ChatCompletionResponse {
        let mut error_response = ChatCompletionResponse {
            model: "Error".to_string(),
            choices: vec![ChatCompletionChoice {
                message: ChatCompletionMessage {
                    role: "system".to_string(),
                    content: String::new(),
                },
            }],
            usage: ChatCompletionUsage {
                total_tokens: 0,
                total_time: 0.0,
            },
        };

        let request = ChatCompletionRequest {
            model,
            messages: vec![ChatCompletionMessage {
                role: "user".to_string(),
                content: message,
            }],
        };

        let response = self
            .http_client
            .post(format!("{}/chat/completions", self.url))
            .header("Authorization", format!("Bearer {}", self.api_key))
            .json(&request)
            .send()
            .await;

        if response.is_err() {
            error_response.choices[0].message.content = "Request Error".to_string();
            return error_response;
        }

        let response_completion = response.unwrap().json::<ChatCompletionResponse>().await;

        if response_completion.is_err() {
            error_response.choices[0].message.content = "Response Parsing Error".to_string();
            return error_response;
        }

        response_completion.unwrap()
    }
}

pub static GROQ_CLIENT: LazyLock<Groq> = LazyLock::new(|| Groq::new(CONFIG.groq_api_key.clone()));
